package conex

  1. Overview
  2. Docs

Module Conex_cryptoSource

Crypto primitives

Implementations are provided in Conex_nocrypto and Conex_openssl.

Verification

Sourcetype verification_error = [
  1. | `InvalidBase64Encoding
  2. | `InvalidSignature
  3. | `InvalidPublicKey
]

Potential error case when verifying a signature

Sourceval pp_verification_error : verification_error Conex_utils.fmt

pp_verification_error is a pretty printer for verification_error.

Sourcemodule type VERIFY = sig ... end

The verification module type

Sourcemodule type VERIFY_BACK = sig ... end

The verification backend, to be implemented by a crypto provider

Instantiation.

Signing

Sourcemodule type SIGN = sig ... end

The signing module type

Sourcemodule type SIGN_BACK = sig ... end

The signing backend, to be implemented by a crypto provider.

Instantiation.