package conex

  1. Overview
  2. Docs

Module Conex_resource.TeamSource

A team consists of a group of authors. Team members can dynamically join and leave.

Sourcetype t = private {
  1. created : Conex_utils.Uint.t;
  2. counter : Conex_utils.Uint.t;
  3. wraps : Conex_utils.Uint.t;
  4. name : identifier;
  5. members : Conex_utils.S.t;
}

The record for a team: a header and a set of members.

pp is a pretty printer.

t ~counter ~wraps ~members created id is a constructor for a team.

Sourceval equal : t -> t -> bool

equal t t' is true if the set of members is equal and the name is equal.

Sourceval of_wire : Wire.t -> (t, string) result

of_wire w converts w to a team or error.

Sourceval wire : t -> Wire.t

wire t is the wire representation of t.

Sourceval add : t -> identifier -> t

add t id adds id to team.

Sourceval remove : t -> identifier -> t

remove t id removes id from team.

Sourceval prep : t -> t * bool

prep t prepares increments t.counter. Returns the carry bit as second component. Used after a batch of changes.